@@ -15,7 +15,7 @@ our @EXPORT_OK = ();
 
 our @EXPORT = ();
 
-our $VERSION = '0.08';
+our $VERSION = '0.11';
 
 require XSLoader;
 XSLoader::load('Unicode::Casing', $VERSION);
@@ -25,6 +25,9 @@ XSLoader::load('Unicode::Casing', $VERSION);
 # into this list.
 my @_function_list;
 
+our @recursed;
+local @recursed;
+
 # The way it works is that each function that is overridden has a
 # reference stored to it in the array.  The index in the array to it is
 # stored in %^H with the key being the name of the overridden function,
@@ -38,6 +41,23 @@ my @_function_list;
 # functions itself calls a casing function, as happens with Unicode
 # table look-ups.)
 
+my $fc_glob;
+
+# This is a work-around, suggested by Matt S. Trout, to the problem that
+# CORE::fc() is a syntax error on Perls prior to v5.15.8.  We have to avoid
+# compiling that expression on those Perls, but we want the compile-time
+# version of it on Perls that handle it.  Another solution would be to put it
+# in a sub module that is loaded with a 'use if,'.  We want CORE:: to get the
+# official version.  We can't do a string eval or otherwise defer this to
+# runtime, because by the time _dispatch is called, the op has been replaced,
+# and we would get infinite recursion.
+# Actually, I'm not sure the CORE:: is actually needed at all, but am leaving
+# it in just to be safe.
+BEGIN {
+    no strict;
+    $fc_glob = \*{"CORE::fc"} if $^V ge v5.15.8;
+}
+
 sub _dispatch {
     my ($string, $function) = @_;
 
@@ -59,14 +79,18 @@ sub _dispatch {
 
     my $index = $hints_hash_ref->{$key};
 
-    if (! defined $index) { # Not overridden
+    if (! defined $index # Not overridden
+        || defined $recursed[$index])
+    {
         return CORE::uc($string) if $function eq 'uc';
         return CORE::lc($string) if $function eq 'lc';
         return CORE::ucfirst($string) if $function eq 'ucfirst';
         return CORE::lcfirst($string) if $function eq 'lcfirst';
-        return;
+        return &$fc_glob($string) if $function eq 'fc';
     }
 
+    local $recursed[$index] = $string;
+
     # Force scalar context and returning exactly one value;
     my $ret = &{$_function_list[$index]}($string);
     return $ret;
@@ -92,12 +116,15 @@ sub import {
             croak("Missing CODE reference for $function");
         }
         if (ref $user_sub ne 'CODE') {
-            croak("$user_sub is not a CODE reference");
+            croak("$user_sub (for $function) is not a CODE reference");
         }
         if ($function ne 'uc' && $function ne 'lc'
-            && $function ne 'ucfirst' && $function ne 'lcfirst')
+            && $function ne 'ucfirst' && $function ne 'lcfirst'
+            && ! ($function eq 'fc' && $^V ge v5.15.8))
         {
-            croak("$function must be one of: 'uc', 'lc', 'ucfirst', 'lcfirst'");
+            my $msg = "$function must be one of: 'uc', 'lc', 'ucfirst', 'lcfirst'";
+            $msg .= ", 'fc'" if $^V ge v5.15.8;
+            croak($msg);
         }
         elsif (exists $args{$function}) {
             croak("Only one override for \"$function\" is allowed");
@@ -120,13 +147,13 @@ sub import {
 }
 
 sub unimport {
-    foreach my $function (qw(lc uc lcfirst ucfirst)) {
+    foreach my $function (qw(lc uc lcfirst ucfirst fc)) {
         my $id = $^H{setup_key($function)};
         teardown($function, $id) if defined $id;
     }
     return;
 }
-
+        
 1;
 __END__
 
@@ -140,7 +167,8 @@ Unicode::Casing - Perl extension to override system case changing functions
 
   use Unicode::Casing
             uc => \&my_uc, lc => \&my_lc,
-            ucfirst => \&my_ucfirst, lcfirst => \&my_lcfirst;
+            ucfirst => \&my_ucfirst, lcfirst => \&my_lcfirst,
+            fc => \&my_fc;
   no Unicode::Casing;
 
   package foo::bar;
@@ -151,6 +179,7 @@ Unicode::Casing - Perl extension to override system case changing functions
             lc      => \&_lc,
             ucfirst => \&_ucfirst,
             lcfirst => \&_lcfirst,
+            fc => \&_fc,
         );
     }
     sub unimport {
@@ -160,30 +189,38 @@ Unicode::Casing - Perl extension to override system case changing functions
 =head1 DESCRIPTION
 
 This module allows overriding the system-defined character case changing
-functions.  Any time something in its lexical scope would
-ordinarily call C<lc()>, C<lcfirst()>, C<uc()>, or C<ucfirst()> the
-corresponding user-specified function will instead be called.  This applies to
-direct calls, and indirect calls via the C<\L>, C<\l>, C<\U>, and C<\u>
-escapes in double quoted strings and regular expressions.
+operations.  Any time something in its lexical scope would ordinarily call
+C<lc()>, C<lcfirst()>, C<uc()>, C<ucfirst()>, or C<fc()>, the corresponding
+user-specified function will instead be called.  This applies to direct calls
+(even those prefaced by C<CORE::>), and indirect calls via the C<\L>, C<\l>,
+C<\U>, C<\u>, and C<\F> escapes in double-quoted strings and regular
+expressions.
 
-Each function is passed a string to change the case of, and should return the 
-case-changed version of that string.  Using, for example, C<\U> inside the
-override function for C<uc()> will lead to infinite recursion, but the
-standard casing functions are available via CORE::.  For example,
+Each function is passed a string whose case is to be changed, and should
+return the case-changed version of that string.  Within the function's
+dynamic scope, references to the operation it is overriding use the
+non-overridden version.  For example:
     
  sub my_uc {
     my $string = shift;
     print "Debugging information\n";
-    return CORE::uc($string);
+    return uc($string);
  }
  use Unicode::Casing uc => \&my_uc;
  uc($foo);
 
 gives the standard upper-casing behavior, but prints "Debugging information"
-first.
+first.  This also applies to the escapes.  Using, for example, C<\U> inside
+the override function for C<uc()> will call the non-overridden C<uc()>.
+Since this applies across the dynamic scope, if C<my_uc> calls function C<a>
+which calls C<b> which calls C<c> which calls C<uc>, that C<uc> is the
+non-overridden version.  Otherwise there would be the possibility of infinite
+recursion.  And, it fits with the typical use of these functions, which is to 
+use the standard case change except for a few select characters, as shown in
+the example below.
 
 It is an error to not specify at least one override in the "use" statement.
-Ones not specified use the standard version.  It is also an error to specify
+Ones not specified use the standard operation.  It is also an error to specify
 more than one override for the same function.
 
 C<use re 'eval'> is not needed to have the inline case-changing sequences
@@ -191,7 +228,7 @@ work in regular expressions.
 
 Here's an example of a real-life application, for Turkish, that shows
 context-sensitive case-changing.  (Because of bugs in earlier Perls, version
-5.12 is required for this example to work properly.)
+v5.12 is required for this example to work properly.)
 
  sub turkish_lc($) {
     my $string = shift;
@@ -208,7 +245,7 @@ context-sensitive case-changing.  (Because of bugs in earlier Perls, version
 
     $string =~ s/\x{130}/i/g;
 
-    return CORE::lc($string);
+    return lc($string);
  }
 
 A potential problem with context-dependent case changing is that the routine
@@ -224,6 +261,10 @@ Unicode behavior, the strings must be encoded in utf8 (which the override
 functions can force) or calls to the operations must be within the scope of C<use
 feature 'unicode_strings'> (which is available starting in Perl version 5.12).
 
+Also, note that C<fc()> and C<\F> are available only in Perls starting with
+version v5.15.8.  Trying to override them on earlier versions will result in
+a fatal error.
+
 Note that there can be problems installing this (at least on Windows)
 if using an old version of ExtUtils::Depends. To get around this follow
 these steps:
@@ -246,11 +287,19 @@ force install B::Hooks::OP::PPAddr
 
 See L<http://perlmonks.org/?node_id=797851>.
 
+=head1 BUGS
+
+This module doesn't play well when there are other attempts to override the
+functions, such as C<use subs qw(uc lc ...);> or
+S<C<*CORE::GLOBAL::uc = sub { .... };>>.  Which thing gets called depends on
+the ordering of the calls, and scoping rules break down.
+
 =head1 AUTHOR
 
 Karl Williamson, C<< <khw@cpan.org> >>,
 with advice and guidance from various Perl 5 porters,
-including Paul Evans, Burak Gürsoy, Florian Ragwitz, and Ricardo Signes.
+including Paul Evans, Burak Gürsoy, Florian Ragwitz, Ricardo Signes,
+and Matt S. Trout.
 
 =head1 COPYRIGHT AND LICENSE
 
@@ -259,6 +308,3 @@ Copyright (C) 2011 by Karl Williamson
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.1 or,
 at your option, any later version of Perl 5 you may have available.
-
-
-=cut
